// FILE: cluster0_1a.c - seems to work.
// DATE: 02/27/20 16:50
// AUTH: P.Oh
// DESC: Clustering: count number of objects in image
// VERS: 1.0a: Pocket Handbook pg. 44-45

#include<stdlib.h>
#include<stdio.h>
#include<memory.h>
#include<math.h>

#define pix(Im, x, y)   *(Im->Data + (x)*Im->Cols + (y))
#define WHITE           255 // pg. 94
#define BLACK           0

struct Image {
    int Rows;             // image's number of rows
    int Cols;             // image's number of columns
    unsigned char *Data;  // pointer to image data
}; // end of struct Image

struct coord {
  float x, y;
};

void Img_in(struct Image *Img) {
    FILE *ifile;
    int i;

    // NB: Assumes already know that using a RAW image file 256 x 256 size
    // Open file for binary reading
    ifile = fopen("20x28-clusterDifferentGrayShadesRaw.raw", "rb"); // rb: reading text or binary files

    // Read directly into the image array
    for(i=0; i < Img->Rows; ++i)
        fread(Img->Data + i*Img->Cols, Img->Cols, 1, ifile);

    fclose(ifile);

} // end Img_in


void Img_out(struct Image *Out) {
    FILE *ofile;
    int i;

    // open (or create) a file for writing
    // ofile = fopen("OUTPUT.RAW", "wb");
    ofile = fopen("OUTPUT.RAW", "wb");
    // out the image by rows
    for(i=0; i < Out->Rows; ++i)
        fwrite(Out->Data + i*Out->Cols, Out->Cols, 1, ofile);

    fclose(ofile);
} // end Img_out

cluster(struct Image *In, int clust_size) {
  int n, i, j, k, new_clust;
  float edist, clustx[256], clusty[256];
  int numberOfObjects = 0;
  // initialize cluster count variable n
  n = 0;
  // double-loop thru each pixel of binary image
  for(i=0; i < In->Rows; ++i) {
    for(j=0; j < In->Cols; ++j) {
        if(n==0) {
            // only do for 1st cluster
            if(pix(In, i, j) != WHITE) {
                    n=1; // 1st cluster found
                    // store X and Y coordinates and mark pixel as cluster 1
                    clustx[1] = i;
                    clusty[1] = j;
                    pix(In, i, j) = 1;
            }
        }
        // test for membership in all known clusters
        else if(pix(In, i, j) != WHITE) {
            // marker for new cluster
            new_clust = 0;
            // compute Euclidean distance
            for(k=1; k <= n; ++k) {
                edist = sqrt( (i-clustx[k])*(i-clustx[k]) + (j-clusty[k])*(j-clusty[k]) );
                // test against cluster size desired
                if(edist <= clust_size) {
                    // set pixel to cluster number
                    pix(In, i, j) = (unsigned char)k;
                    new_clust = 1;
                    k = n+1;
                    numberOfObjects++;
                }; // end if
            }; // end for loop k
        }; // end else if
    }; // end for loop j
  }; // end for loop i

  // add a new cluster
  if(new_clust == 0 && n <=255) {
      n = n+1;
      clustx[n] = i;
      clusty[n] = j;
      pix(In, i, j) = (unsigned char)n;
      // pix(In, i, j) = BLACK;
   };

   printf("clust[x] = %3.3f, clust[y] = %3.3f\n", clustx[n], clusty[n]);
   printf("n = %d\n", n);
   printf("k = %d\n", k);
   printf("numberOfObjects = %d\n", numberOfObjects);

} // end cluster function

int main() {

  FILE *ofile;
  struct Image In, Out; // Declare input and output images
  int clusterSize = 3; // smaller object size is 10 pixels. SQRT(10) = 3.16, so use 3
  // Indexes for mask generation section
  // signed char *tmp = NULL;

  // Initialize image parameters
  In.Rows = Out.Rows = 20;
  In.Cols = Out.Cols = 28;
  // Allocate memory for image
  In.Data = (unsigned char *)calloc(In.Rows, In.Cols);
  Out.Data = (unsigned char *)calloc(Out.Rows, Out.Cols);
  ofile = fopen("OUTPUT.RAW", "wb");

  Img_in(&In);
  cluster(&In, clusterSize);
  printf("clusterSize = %d\n", clusterSize);
  Img_out(&In);

  fclose(ofile);
} // end of main
